# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

################################################################################
############# IMPORTAZIONE MODULI  BASE ########################################
################################################################################
import gtk
import os.path
################################################################################
### FINESTRA DI SELEZIONE OPZIONI GENERICA #####################################
################################################################################

class Dialog(gtk.Dialog):
    def __init__(self, titolo, options, buttons=None, op_keys_names=None):
        self.__old_options = options
        if buttons == "apply":
            buttons = (gtk.STOCK_APPLY, gtk.RESPONSE_ACCEPT)
        elif buttons == "save":
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                    gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT)
        else:
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                    gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT)
        # creo una copia del dizionario per opter ripristinare le opzioni base
        self.options = dict()
        self.dialog = gtk.Dialog(titolo, None,
                     gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, buttons)
        self.dialog.set_resizable(False)
        self.dialog.set_position(gtk.WIN_POS_CENTER)
        # Create a orizontal box
        hbox = gtk.HBox(False, 2)
        self.dialog.vbox.pack_start(hbox, False, False, 2)
        hbox.show()
        # Create a vertical box for buttons
        vbox = gtk.VBox(False, 2)
        hbox.pack_start(vbox, False, False, 2)
        vbox.show()
        # Create a separator
        vsep = gtk.VSeparator()
        hbox.pack_start(vsep, False, False, 0)
        vsep.show()
        # Create a vertical box for gtk.RadioButton
        vbox1 = gtk.VBox(False, 2)
        vbox.pack_start(vbox1, False, False, 2)
        vbox1.show()
        # Create a vertical box for gtk.CheckButton
        vbox2 = gtk.VBox(False, 2)
        vbox.pack_start(vbox2, False, False, 2)
        vbox2.show()
        # Create a vertical box for gtk.Entry
        vbox3 = gtk.VBox(False, 2)
        hbox.pack_start(vbox3, False, False, 2)
        vbox3.show()
        # imposta una "radioaction" per eventuali pulsanti di scelta
        radiobutton_group = gtk.RadioButton(group=None, label=None, use_underline=True)
        # set the option dictionary and create the buttons
        for key, value in options.items():
            self.options[key] = value
            if op_keys_names is not None:
                text = op_keys_names.get(key, str(key))
            else:
                text = str(key)
            # se il valore è un intero VERO/FALSO crea un gruppo di gtk.RadioButton
            if type(value) == tuple:
                button = gtk.RadioButton(group=radiobutton_group, label=text, use_underline=True)
                button.set_active(value[0])
                button.connect("toggled", self.callback_radiobutton, key)
                vbox1.pack_start(button, False, False, 2)
                button.show()
            # se il valore è VERO/FALSO usa un gtk.CheckButton
            elif type(value) == bool:
                button = gtk.CheckButton(text.replace("_", " "))
                button.set_active(value)
                button.connect("toggled", self.callback_button, key)
                vbox2.pack_start(button, False, False, 2)
                button.show()
            # se il valore è un file usa un gtk.filechooser
            elif type(value) == str and os.path.exists(os.path.dirname(value)):
                label = gtk.Label("<i>" + text.replace("_", " ") + ":" + "</i>")
                label.set_use_markup(True)
                titolo = ''
                # crea il pulsante e la fin. dialogo x scelta file
                dialog = gtk.FileChooserDialog(titolo, None, 
                              gtk.FILE_CHOOSER_ACTION_OPEN, (gtk.STOCK_CANCEL, 
                              gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
                FilechooserButton = gtk.FileChooserButton(dialog)
                FilechooserButton.connect("file-set", self.callback_filechooser, key)
                vbox3.pack_start(label, False, False, 2)
                vbox3.pack_start(FilechooserButton, False, False, 2)
                # imposta i filtri per i file
                filter = gtk.FileFilter()
                filter.set_name("All files")
                filter.add_pattern("*")
                dialog.add_filter(filter)
                # filtro per i file database sqlite3
                filter1 = gtk.FileFilter()
                filter1.set_name("Database SQLITE")
                filter1.add_mime_type("application/x-sqlite3")
                filter1.add_pattern("*.db")
                dialog.add_filter(filter1)
                dialog.set_filter(filter1)
                # stabilisce se stiamo gestendo un file o una directory
                if os.path.isfile(value):
                    dialog.set_filename(value)
                    dialog.select_filename(value)
                elif os.path.isdir(value):
                    dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
                    dialog.set_current_folder(value)
                else: 
                    dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
                    dialog.set_current_folder(value)
                # mostra la finestra di dialogo
                label.show()
                FilechooserButton.show()
            # se il valore è una stringa usa un gtk.entry
            elif type(value) == str:
                label = gtk.Label("<i>" + text.replace("_", " ") + ":" + "</i>")
                label.set_use_markup(True)
                entry = gtk.Entry()
                entry.set_text(value)
                entry.set_icon_from_stock(1, gtk.STOCK_SAVE)
                entry.connect("activate", self.callback_entry, key)
                entry.connect("icon-press", self.callback_entry_icon, key)
                vbox3.pack_start(label, False, False, 2)
                vbox3.pack_start(entry, False, False, 2)
                label.show()
                entry.show()
            # se il valore è un numero intero utilizza uno SpinButton
            elif type(value) == int:
                label = gtk.Label("<i>" + text.replace("_", " ") + ":" + "</i>")
                label.set_use_markup(True)
                adj = gtk.Adjustment(value=value, lower=0, upper=value*10, step_incr=1, page_incr=0, page_size=0)
                button = gtk.SpinButton(adjustment=adj, climb_rate=1, digits=0)
                button.connect("value_changed", self.callback_spinbutton, key)
                hboxSpin = gtk.HBox(False, 2)
                hboxSpin.pack_start(label, False, False, 2)
                hboxSpin.pack_start(button, False, False, 2)
                vbox2.pack_start(hboxSpin, False, False, 2)
                label.show()
                button.show()
                hboxSpin.show()
            # se il valore è una lista utilizza una gtk.entry
            elif type(value) == list:
                stringa = str()
                for data in value:
                    stringa += data + ","
                # imposta l'etichetta e il valore della gtk.entry
                label = gtk.Label("<i>" + text.replace("_", " ") + ":" + "</i>")
                label.set_use_markup(True)
                entry = gtk.Entry()
                entry.set_text(stringa)
                entry.set_icon_from_stock(1, gtk.STOCK_SAVE)
                entry.connect("icon-press", self.callback_entry_list, key)
                vbox3.pack_start(label, False, False, 2)
                vbox3.pack_start(entry, False, False, 2)
                label.show()
                entry.show()
################################################################################
# Avvio dell finestra di dialgo ################################################
################################################################################
    def run(self):
        response = self.dialog.run()
        if response == gtk.RESPONSE_ACCEPT:
            options = self.options
        elif response == gtk.RESPONSE_CANCEL:
            options = self.__old_options
        elif response == gtk.RESPONSE_CLOSE:
            options = self.__old_options
        else: options = self.__old_options
        self.dialog.destroy()
        return options
################################################################################
# Funzioni richiamate quando di immettono dati nell'interfaccia grafica ########
################################################################################
    def callback_radiobutton(self, widget, data=None):
        if widget.get_active():
            self.options[data] = (1,)
        else: self.options[data] = (0,)
    def callback_button(self, widget, data=None):
        self.options[data] = widget.get_active()
    def callback_spinbutton(self, widget, data=None):
        self.options[data] = int(widget.get_value())
    def callback_entry(self, widget, data=None):
        self.options[data] = widget.get_text()
    def callback_entry_icon(self, widget,icon_pos, event, data=None):
        self.options[data] = widget.get_text()
    def callback_filechooser(self, widget, data=None):
        self.options[data] = widget.get_filename()
    def callback_entry_list (self, widget, icon_pos, event, data=None):
        self.options[data] = widget.get_text().split(",")
        for par in self.options[data]:
            par = par.rstrip(" ").lstrip(" ")
            if par == "": self.options[data].remove(par)
